
from collections import defaultdict
import Bio.SeqIO
from Bio import pairwise2
from Bio.SubsMat import MatrixInfo as matlist
from glob import glob
import cogent
import os
import numpy as np
import hcluster
from hcluster import pdist, single, linkage, dendrogram
from cogent.cluster.UPGMA import upgma
from cogent.phylo import distance, nj

def parseBlastResults(bfile):
    idx=10
    ddict = {}
    with open(bfile) as ifile:
        for l in ifile:
            toks = l.rstrip().split()
            if toks[0] not in ddict:
                ddict[toks[0]] = { toks[1] : float(toks[idx]) }
            else:
                ddict[toks[0]][toks[1]] = float(toks[idx])
    return ddict

def readTrees(tfname):
    trees = {}
    
    for i,fn in enumerate(glob(os.path.sep.join([tfname,'*.nwk.rooting*']))):
        print(fn)
        T = cogent.LoadTree(fn)
        for n in T.preorder():
            if n.Name.find('LOST') != -1:
                n.Name = "{0}.TREE{1}".format(n.Name,i)
            n.Name = n.Name.upper()
        trees[i] = T
        
    return trees

def main():
    #tfname = '/Users/rob/Dropbox/ViralPPIs/dataOut_June17/rearranged'
    tfname = '/Users/rob/SoftwareStaging/Parana2Data/HerpesPPIs/trees/rearranged'
    trees = readTrees(tfname)
    print(trees)
    sfile = '/Users/rob/Dropbox/ViralPPIs/allFasta/allseqs.fasta'
    seqs = { s.id: s for s in Bio.SeqIO.parse(sfile, 'fasta') }

    smap = defaultdict(list)
    for num, t in trees.iteritems():
        for tip in t.getTipNames():
            if tip.find('LOST') == -1:
                smap[num].append( seqs[tip] )
    
    D = np.zeros((len(smap), len(smap)))

    bfile = '/Users/rob/SoftwareStaging/Parana2Data/HerpesNew/seqs/allpairs.res'
    sddict = parseBlastResults(bfile)
    distdict = {}
    for i in xrange(len(smap)):
        for j in xrange(i+1, len(smap)):
            bestscore = np.inf
            denom = 0

            for s1 in smap[i]:
                for s2 in smap[j]:
                    
                    if s1.id == s2.id:
                        print(trees[i])
                        print(trees[j])
                        print([ s.id for s in smap[i] ])
                        print([ s.id for s in smap[j] ])
                        raise "Hell"
                    if s2.id in sddict[s1.id]:
                        bestscore = min(bestscore, sddict[s1.id][s2.id])
                        denom += 1
                        D[i][j] += sddict[s1.id][s2.id]
                        D[j][i] += sddict[s1.id][s2.id]

            distdict[(str(i), str(j))] = D[i][j]/float(denom)
            #distdict[(str(i), str(j))] = bestscore
    print(distdict)
    myclust = upgma(distdict)
    print(myclust.asciiArt())

    origTips = [ t for t in myclust.tips() ]

    for ot in origTips:
        parent = ot.Parent
        for i, c in enumerate(parent.Children):
            if c == ot:
                otIdx = int(ot.Name)
                parent.removeNode(ot)
                parent.insert(i, trees[otIdx])

    myclust.nameUnnamedNodes()
    print(myclust.asciiArt())
    

    species = set(['MCMV', 'HSV1', 'EBV', 'VZV', 'KSHV', 'A*', 'G*', 'BG*', 'ABG*'])
    A = frozenset( ['HSV1', 'VZV'] )
    G = frozenset( ['EBV', 'KSHV'] )
    BG = G.union( frozenset(['MCMV']) )
    ABG = BG.union(A)

    def getSubSpecies(tips):
        spec = set([])
        for t in tips:
            for s in species:
                if t.upper().find(s) != -1 :
                    spec.add(s)

        return frozenset(spec)

    def subSpecies(x):
        specs = getSubSpecies(x.get_leaf_names())
        ns = set()
        for s in specs:
            if s == 'A*':
                ns |= A
            elif s == 'G*':
                ns |= G
            elif s == 'BG*':
                ns |= BG
            elif s == 'ABG*':
                ns |= ABG
            else:
                ns.add(s)

        ns = frozenset(ns)
        if len(ns) == 1:
            return list(ns)[0]

        else:
            if ns == A:
                return 'A'
            elif ns == G:
                return 'G'
            elif ns == BG:
                return 'BG'
            elif ns == ABG:
                return 'ABG'


    import ete2
    ET = ete2.PhyloTree( myclust.getNewick(with_distances=True, escape_name=False) )
    import sys

    def isLost(x):
        return x.name.find('LOST') != -1

    def isAncestral(x):
        return x.name.find('ancestral') != -1

    for i,x in enumerate(ET.traverse('postorder')):
        specname = subSpecies(x)
        x.add_feature('species', specname) 
        if x.name == "NoName":            
            x.add_feature("name", "ancestral{0}".format(i))

        if isLost(x) or isAncestral(x):
            x.add_feature("name", x.name+'_'+specname)
        x.set_species_naming_function( lambda x: x.split('_')[-1] )
        x.add_feature("S", x.species)
        if (x.species != subSpecies(x)):
            print("CHA")
            print(x.name, x.species, subSpecies(x))
            sys.exit(1)
    ET.add_feature('name', 'root_ABG')
    ET.add_feature('species', 'ABG')
    print(ET)
    ET.write(features=["species","name","support", "S"], outfile="herpes_upgma_avg.nwk", format=1)
    #myclust.writeToFile('herpes_upgma.xml', with_distances=True, format='xml')

    # L = single(D)
    # LTree = hcluster.to_tree(L)

    # trees[0].rooting

    # def postOrder(r):
    #     if not r.is_leaf():
    #         lid = r.get_left().id
    #         rid = r.get_right().id
    #         print(trees[lid], trees[rid])
    #         postOrder( r.get_left() )
    #         postOrder( r.get_right() )
    #     print( r.id)

    # postOrder(LTree)
    # l = LTree.pre_order()
    # print(l)
    # print(LTree)
    # dendrogram(L)

    # # print D
    # # print D.shape
    # # print np.where(D == 1000)
    # from matplotlib import pyplot as plt
    # # print(D.flatten())
    # # plt.hist(D.flatten())
    # plt.show()


if __name__ == "__main__":
    main()