"""

Usage: CompareDuplicationOrders.py -x <x> -y <y>

Options
  -x <x>   First duplication order
  -y <y>   Second duplication order
"""
from docopt import docopt
import numpy as np
import scipy as sp
import scipy.stats

def parseOrdFile(fname):

    ordDict = {}

    with open(fname, 'rb') as ifile:
        for l in ifile:
            toks = l.rstrip().split()
            p1 = toks[0]
            p2 = toks[1]
            oval = int(toks[2])

            if p1 > p2:
                oval *= -1
            p1, p2 = sorted((p1, p2))

            ordDict[(p1, p2)] = oval

    return ordDict

def taub( keysInBoth, d1, d2 ):
    from collections import defaultdict
    def r0(): return 0.0
    concordant = 0
    discordant = 0
    t = defaultdict(r0)
    u = defaultdict(r0)

    for k in keysInBoth:
        v1, v2 = d1[k], d2[k]
        hadTie = False
        if v1 == 0:
            t[k[0]] += 1
            t[k[1]] += 1
            hadTie = True
        if v2 == 0:
            u[k[0]] += 1
            u[k[1]] += 1
            hadTie = True

        if not hadTie:
            if v1 == v2:
                concordant += 1
            elif v1 != v2:
                discordant += 1
            else:
                raise "Hell"

    N = len(keysInBoth)
    ties1 = sum( x*(x-1) / 2.0 for x in t.values() )
    ties2 = sum( x*(x-1) / 2.0 for x in u.values() )

    import math
    n = math.ceil(np.sqrt(2.0*N))

    v0 = n*(n-1) * (2*n+5)
    vt = sum( x*(x-1) * (2*x + 5) for x in t.values() )
    vu = sum( x*(x-1) * (2*x + 5) for x in u.values() )
    v1 = sum( x*(x-1) * sum( y*(y-1) / (2.0*n*(n-1)) for y in u.values() ) for x in t.values() )
    v2 = sum( x*(x-1) * (x-2) * sum( y*(y-1)*(y-2) / (9*n*(n-1)*(n-2)) for y in u.values()) for x in t.values() )

    v = (v0 - vt - vu) / 18.0 + v1 + v2

    zb = (concordant - discordant) / np.sqrt(v)
    
    p = sp.stats.norm.sf(zb, loc=0.0, scale=1.0)

    ktb = (concordant - discordant) / np.sqrt((N-ties1) * (N-ties2))
    print("Kendall-tau b: {0}".format(ktb))
    print("p-value: {0}".format(p))
    return ktb

def main(args):

    ord1File = args['-x']
    ord2File = args['-y']

    d1 = parseOrdFile(ord1File)
    d2 = parseOrdFile(ord2File)

    keysInBoth = set(d1.keys()).intersection(set(d2.keys()))
    
    tb = taub(keysInBoth, d1, d2)

    concordant = 0
    discordant = 0
    N = 0
    for k in keysInBoth:
        v1, v2 = d1[k], d2[k]

        if v1 == 0 or v2 == 0: continue

        if v1 == v2:
            concordant += 1
        elif v1 != v2:
            discordant += 1
        else:
            raise "Hell"

        N += 1

    import math
    n = math.ceil(np.sqrt( 2.0 * N ))
    print(n)

    variance = float(2*(2*n+5)) / float(9*n*(n-1))
    stddev = np.sqrt(variance)

    kt = (concordant - discordant) / (float(N))
    
    p = sp.stats.norm.sf(kt, loc=0.0, scale=stddev)
    p2 = sp.stats.norm.sf(tb, loc=0.0, scale=stddev)
    print(p,p2)
    print("concordant = {0}, discordant = {1}".format(concordant, discordant))
    print("Kendall-tau = {0}".format(kt))




if __name__ == "__main__":

    args = docopt(__doc__, version='CompareDuplicationOrders v1.0')
    main(args)