import networkx as nx
import ete2
import sys
from collections import namedtuple, defaultdict
import itertools

def main():
    P = ete2.Phyloxml()
    P.build_from_file( sys.argv[1] )
    T = P.phylogeny[0]

    root = T
    #root.convert_to_ultrametric( 10 )

    nodeinfo = namedtuple('NodeInfo', ['name', 'species', 'duptime', 'node'])
    nodeTimes = defaultdict(list)
    for n in root.traverse('postorder'):
        if n != root and not n.is_leaf():
            time = root.get_distance(n)
            species = n.phyloxml_clade.taxonomy[0].scientific_name
            nodeTimes[species].append(nodeinfo(n.name, species, time, n))
    
    def isLost(x):
        return x.name.find('LOST') != -1

    def spec(n):
        return n.phyloxml_clade.taxonomy[0].scientific_name

    ## Group nodes by species
    with open(sys.argv[2],'wb') as ofile:
        for species, snodes in nodeTimes.iteritems():
            for n1,n2 in itertools.combinations(snodes,2):
                
                n1, n2 = sorted((n1,n2))
                
                if isLost(n1) or isLost(n2): continue
                #if spec(n1.node.children[0]) != n2.species or spec(n2.node.children[0]) != n1.species: continue

                if n1.duptime < n2.duptime:
                    ofile.write("{0}\t{1}\t{2}\n".format(n1.name, n2.name, '-1'))
                elif n1.duptime > n2.duptime:
                    ofile.write("{0}\t{1}\t{2}\n".format(n1.name, n2.name, '1'))
                else:
                    ofile.write("{0}\t{1}\t{2}\n".format(n1.name, n2.name, '0'))



if __name__ == "__main__":
    main()