"""

Usage: PlotOverlappingHistograms.py (-i <input> -n <name>)... [-b <bins>]

Options:
  -i <in>       Input file
  -n <name>     Name for dataset
  -b <bins>     Number of bins for histograms [default: 50]
"""
from docopt import docopt
import numpy as np
import sys
from matplotlib import pyplot


def getRanks(ifname):

    ranks = []
    with open(ifname, 'rb') as ifile:
        for l in ifile:
            ranks.append(float(l.strip().split()[-1]))

    return np.array(ranks)


def main(args):

    inputs = args['-i']
    names = args['-n']
    nbins = int(args['-b'])
    ranks = {}
    for name, infile in zip(names, inputs):
        ranks[name] = getRanks(infile)

    for name in names:
        if name == 'SOPH':
            #[ '/' | '\\' | '|' | '-' | '+' | 'x' | 'o' | 'O' | '.' | '*' ] 
            pyplot.hist(ranks[name], label=name, bins=nbins, alpha=0.45, histtype='stepfilled', hatch='\\')
        else:
            pyplot.hist(ranks[name], label=name, bins=nbins, alpha=0.3, histtype='stepfilled')

    params = {'font.size': 20, 'lines.linewidth': 3.0}

    pyplot.rcParams.update(params)

    pyplot.xlabel('Relative Rank of Imputed Edge')
    pyplot.ylabel('# of Imputed Edges with Given Relative Rank', labelpad=25)
    pyplot.grid(axis='both')
    pyplot.legend()
    pyplot.show()

if __name__ == "__main__":
    args = docopt(__doc__, version="Plot Overlapping Histograms v1.0")
    main(args)
